<?php

	class apiUserClass  {
		
		var $id;
		var $name;
		var $api_key;
		
		var $db = -1;
		
		
		
		/*---------------------------------------------------------------------
		 * Function		: apiUserClass::__construct()
		 * 
		 * Initialize the class.  Optional arguments are $db and $user_id as 
		 * follows: 
		 * 
		 * $db : If defined then apiUserClass::db will be set to this and it
		 * 		 will be used as the database resource for database trans
		 * 		 actions within this object.
		 * 
		 * $user_id :	This can be either an $id or an $api_class.  If it is
		 * 		 set then the class will attempt to load this user from the 
		 * 		 database matching a user record where either the id or api_class
		 * 		 matches $user_id.  This allows flexible loading based on either
		 * 		 identifier since api_key and id are very different data types 
		 * 		 and therefore should never conflict.
		 */
		
		function __construct( $db="", $user_id = "") {
			//  Trigger a commit
			//  And another
			
			if ( $db != "" ) {
				$this->setDB( $db );
			}
			
			if ( $user_id != "" ) {
				$this->load( $user_id );
			}
		}
		
		
		/*---------------------------------------------------------------------
		 * Function		: apiUserClass::load()
		 * 
		 * Loads a user record with either an id or api_key column that matches
		 * $user_id.
		 * 
		 * Returns FALSE if the given $user_id returned no matches.  Otherwise
		 * the column values are loaded into object properties and the function
		 * returns TRUE.
		 * 
		 */
		function load( $user_id ) {
			
			//  Create a template SQL query and prepare it.
			//
			$sql_query = "
					
					SELECT		id, name, api_key, date_added
					
					FROM		webapi_users
						AS		users
					
					WHERE		users.id = :user_id
						OR		users.api_key = :user_id

					LIMIT		1;
					
					";
		
			$sql = $this->db->prepare( $sql_query );
			
			//  Bind parameters and execute the statement.
			//
			$sql->bindParam( ":user_id", $user_id );
			$sql->execute();
			
			//  Get all results into an array.
			//
			$a_results = $sql->fetchAll();

			//  If the number of rows returned is 0 then the match
			//  failed.
			//
			if ( count( $a_results ) == 0 ) {
				$success = FALSE;
			} 
			//
			//  Otherwise load returned values into object properties.
			//
			else {
				$a_user = $a_results[ 0 ];
			
				$this->id 		= $a_user[ 'id'];
				$this->name 	= $a_user[ 'name'];
				$this->api_key 	= $a_user[ 'api_key'];

				$success = TRUE;
			}
			
			return( $success );
		}
		
		
		function setDB( $db ) {
			$this->db = $db;
		}
		
	}

	?>