<?php

require_once 'class.trip_class.inc.php';
require_once 'bo2_trip_data.class.php';

class dbClass extends PDO {

		var $schedule_db	= "salestool";
		var $salestool_db	= "salestool";
		var $paxtool_db		= "paxtool_siren";
		var $bms_db			= "bms";

		var $error_message;			//  If an error occurs in data retrieval this will contain the error message.



		/**
		 * Function		: isValidKey()
		 *
		 * Determine whether the API key is valid.
		 *
		 *
		 * @param string $api_key
		 * @return boolean
		 */
		function isValidKey( $api_key="" ) {

			$sql_query = "

					SELECT		COUNT(*)			AS api_matches

					FROM		webapi_users
						AS		users

					WHERE		users.api_key = :api_key;

					";

			$sql = $this->prepare( $sql_query );
			$variables = array( ':api_key'=>$api_key );
			$sql->execute( $variables );

			$is_valid_key = ( (int)$sql->fetchColumn() == 1 );

			if ( DEBUG == true ) {
				$is_valid_key = true;
			}

			return( $is_valid_key );
		}




		/**
		 * Function		: getVesselList()
		 *
		 * Get a list of operating vessels.
		 *
		 */
		function getVesselList( ) {

			$sql_query = "

					SELECT 		vessels.vessel_name		AS vessel_name

					FROM		{$this->schedule_db}.ml_vessels
						AS		vessels

					WHERE 		vessels.vessel_end_operation >= NOW()

					ORDER BY	vessel_name;

					";

			$sql = $this->prepare( $sql_query );

			try {
				$sql->execute( null );
				$a_vessels = $sql->fetchAll( PDO::FETCH_OBJ );
			}

			catch ( PDOException $e ) {
				//
				// TODO: Add proper exception handler logic.
				//
				$a_vessels = array();
			}

			return( $a_vessels );
		}


		/**
		 * Function			: getLastTripDate()
		 * s
		 * @param string $vessel
		 * @return mixed
		 */
		function getLastTripDate( $vessel="" ) {

			//  If a vessel is defined, match either the name or the key.
			//
			$vessel_filter = ( $vessel == "" ) ? "1" : "( vessels.vessel_key = :vessel ) OR ( vessels.vessel_name = :vessel)";

			$sql_query = "

					SELECT		trips.trip_start_date		AS trip_date

					FROM		{$this->schedule_db}.ml_trips
						AS		trips

					JOIN		{$this->schedule_db}.ml_vessels
						AS		vessels
						ON		vessels.vessel_key = trips.trip_vessel

					WHERE		{$vessel_filter}

					ORDER BY	trip_date DESC

					LIMIT 		1;

					";

			$sql = $this->prepare( $sql_query );

			$a_params = array(
				':vessel'=>$vessel
				);

			try {
				$sql->execute( $a_params );
				$last_trip_date = $sql->fetchColumn();
			}

			catch( PDOException $e ) {
				//
				// TODO: Add proper exception handler logic.
				//
				$last_trip_date = null;
			}

			return( $last_trip_date );
		}




		// Get a list of the cabin classes in use on a given vessel between two dates.
		//
		function getActiveCabinClasses( $vessel=null, $start_date=null, $end_date=null ) {

			$a_params = array();

			//  If a vessel is defined, match either the name or the key.
			//
			if( $vessel == null ) {
				$vessel_filter = "1";
			}
			else {
				$vessel_filter = "( vessels.vessel_key = :vessel ) OR ( vessels.vessel_name = :vessel)";
				$a_params[ ':vessel' ] = $vessel;
			}

			//  Clean up the start and end dates if they were provided.
			//
			$start_date = ( $start_date == null )
				? date( "Y-m-d" )
				: date( "Y-m-d", strtotime( $start_date ));

			$end_date 	= ( $end_date == null )
				? date( "2099-12-31" )
				: date( "Y-m-d", strtotime( $end_date ));


			$statement = "

					SELECT			classes.class_key			AS class_key,
									classes.class_name			AS class_name,
									classes.class_sort_order	AS class_sort_order

					FROM			{$this->schedule_db}.ml_classes
					  AS			classes

					JOIN			{$this->schedule_db}.ml_vessels
					  AS			vessels
					  ON			vessels.vessel_key = classes.class_vessel

					WHERE			{$vessel_filter}
					  AND			( classes.class_active_from <= :start_date OR classes.class_active_to >= :end_date )
					  AND 			classes.class_is_public

					GROUP BY 		classes.class_key

					ORDER BY		classes.class_sort_order;

					";

			$sql = $this->prepare( $statement );

			$a_params = array_merge(
					$a_params,
					array(
						':start_date'		=> $start_date,
						':end_date'			=> $end_date
						)
					);

			try {
				$sql->execute( $a_params );
				$a_classes = $sql->fetchAll( PDO::FETCH_OBJ );
			}
			catch ( PDOException $e ) {
				echo "Exception!\b\b{$e}\n\n";
				$a_classes = array();
				echo $e;
			}

			return( $a_classes );
		}





	/**
	 * Function		: getVesselBookingManager()
	 *
	 * @param unknown $vessel
	 *
	 * Determine the booking manager for a given vessel.
	 *
	 */
	function getVesselBookingManager( $vessel ) {

		$query = "
				SELECT		vessels.ves_booking_manager

				FROM		`{$this->salestool_db}`.st_vessels
				  AS		vessels

				WHERE		vessels.ves_name = :vessel

				LIMIT		1
				";

		$sql = $this->prepare( $query );

		$a_params = array(
			":vessel"		=> $vessel
			);

		try {
			$sql->execute( $a_params );
			$booking_manager = $sql->fetchColumn();
		}
		catch ( PDOException $e ) {
			$booking_manager = null;
		}

		return( $booking_manager );
	}


	/**
	 * Function		: getVesselInfo()
	 *
	 * @param $vessel_name
	 * @return bool|mixed
	 *
	 * Return some basic information about the given vessel.  Used to determine whether the vessel
	 * has multiple classes, which fleet it is in, etc, so that we can decide which query logic
	 * to use.
	 *
	 */
		function getVesselInfo( $vessel_name ) {

			$query = "
				SELECT		vessels.ves_name					AS name,
							vessels.ves_fleet					AS fleet,
							vessels.ves_booking_manager			AS booking_manager,
							vessels.ves_multi_class				AS is_multi_class,
							vessels.ves_first_published_date	AS first_published_date,
							vessels.ves_latest_published_date	AS last_published_date

				FROM		`{$this->salestool_db}`.st_vessels
					AS 		vessels

				WHERE		vessels.ves_name = :vessel_name

				LIMIT		1
				";

			$sql = $this->prepare( $query );

			$a_params = array(
				":vessel_name"		=> $vessel_name
			);

			try {
				$sql->execute( $a_params );
				$vessel_info = $sql->fetchObject();
			}

			catch( PDOException $e ) {
				$vessel_info = FALSE;
			}

			return( $vessel_info );
		}


	/**
	 * Function		: getVesselTrips()
	 *
	 * @param string $vessel
	 * @param string $start_date
	 * @param string $end_date
	 * @param string $classes
	 * @param string $currency
	 * @return array
	 *
	 * Get a list of trips scheduled on a vessel.
	 *
	 */
		function getVesselTrips( $vessel="", $start_date=null, $end_date=null, $classes="", $currency="" ) {

			// Clean up the start and end dates.
			//
			$start_date = ( $start_date == null )
					? date( "Y-m-d")
					: date( "Y-m-d", strtotime( $start_date ));

			$end_date = ( $end_date == null )
					? "2099-12-31"
					: date( "Y-m-d", strtotime( $end_date ));

			$vessel_info = $this->getVesselInfo( $vessel );

			if ( $vessel_info == false ) {
				$a_trips = array();
			}
			else {

				switch ( $vessel_info->booking_manager ) {
				    
					case 'BMS' :
						$a_trips = $this->getVesselTripsFromBMS(
							$vessel_info,
							$start_date,
							$end_date,
							$classes,
							$currency
						);
						break;

					case 'Paxtool' :

						//  Check the fleet because we need to handle BO2 vessels
						//  differently.
						//
						switch( strtolower( $vessel_info->fleet )) {

							case 'blue o two' :
								$a_trips = $this->getVesselTripsFromPaxtoolForBO2(
									$vessel_info,
									$start_date,
									$end_date,
									$classes,
									$currency
								);								
								break;

							default :
								$a_trips = $this->getVesselTripsFromPaxtool(
									$vessel_info,
									$start_date,
									$end_date,
									$classes,
									$currency
								);
								break;
						}
						break;


					default :
						//  No other options to handle at this point.  Just set the result to FALSE.
						$a_trips = FALSE;
						break;
				}
			}

			return( $a_trips );
		}


	/**
	 * Function		: getVesselClassesFromBMS()
	 *
	 * @param $vessel_name
	 * @param $start_date
	 * @param $end_date
	 * @return array|bool
	 *
	 * Returns an array of class names available on the given vessel on trips departing
	 * between the start and end dates.
	 *
	 */
	function getVesselClassesFromBMS( $vessel_name, $start_date, $end_date ) {

		// TODO-Don: Get the classes to sort by lowest price available, highesty price available and then class name.  This
		// will list lowest priced cabins first.  If cabin A and B are the same price on trip 1, but for trip 2 the
		// price of cabin A is increased while cabin B remains the same, then cabin B will list before cabin A.

		$query = "
			SELECT			trip_cabin.cabin_type						AS class_name

			FROM			`{$this->bms_db}`.trip_cabin
				AS			trip_cabin

			JOIN 			`{$this->bms_db}`.trip
				AS			trip
				ON 			trip.id = trip_cabin.trip_id

			JOIN 			`{$this->bms_db}`.trip_template_profile
				AS			trip_template_profile
				ON 			trip_template_profile.trip_template_id = trip.trip_template_id

			JOIN			`{$this->bms_db}`.boat_profile
				AS			boat_profile
				ON			boat_profile.boat_id = trip.boat_id

			WHERE			boat_profile.name = :vessel_name

			GROUP BY 		class_name

			";

		$sql = $this->prepare( $query );

		$a_params = array(
			":vessel_name"		=> $vessel_name,
			":start_date"		=> $start_date,
			":end_date"			=> $end_date
		);

		try {
			$sql->execute( $a_params );

			$a_classes = array();
			while( $row = $sql->FetchObject() ) {
				$a_classes[ $row->class_name ] = $row->class_name;
			}
		}

		catch ( PDOException $e ) {
			$a_classes = array();
		}

		return( $a_classes );

	}


	/**
	 * Function		: getVesselTripsFromBMS()
	 *
	 * @param $vessel_info
	 * @param $start_date
	 * @param $end_date
	 * @param $classes
	 * @param $currency
	 * @return mixed
	 *
	 * Get a list of trips, with pricing and availability for each from the BMS database.
	 *
	 */
	function getVesselTripsFromBMS(
		$vessel_info,
		$start_date,
		$end_date,
		$classes,
		$currency
		){

		//  Get a list of classes available on this vessels during the given operating dates.
		//
		$a_classes = $this->getVesselClassesFromBMS(
				$vessel_info->name,
				max( $start_date, $vessel_info->first_published_date),
				min( $end_date, $vessel_info->last_published_date )
				);

		//  Sub-qurey to count total available spaces on a given trip.
		//
		$count_trip_availability = "
			SELECT		COUNT(*)

			FROM		`{$this->bms_db}`.seat_availability
				AS		avail_seat

            JOIN        `{$this->bms_db}`.trip_cabin
              AS        avail_cabin
              ON        avail_cabin.id = avail_seat.trip_cabin_id

            WHERE		avail_seat.trip_id = trip.id
            	AND		( avail_seat.status = 'OPEN' OR avail_seat.is_buy_back = '1')
			";


		//  Primary query to get vessel trips from BMS database.
		//
		//  2017-03-17:DABD (RM #7902)
		//		Update to filter out Dry Docks and non-Itinerary trips.
		//
		$statement = "
			SELECT			boat_profile.name							AS vessel_name,
							trip.code									AS trip_code,
							trip.date_from								AS trip_start_date,
							trip.date_to								AS trip_end_date,
							DATEDIFF( trip.date_to, trip.date_from )	AS trip_nights,
							trip_template_profile.name					AS trip_itinerary,
							trip_template_profile.name					AS trip_itinerary_long,
							trip.embarkation							AS trip_embarkation_point,
							trip.disembarkation							AS trip_disembarkation_point,
							0											AS price_class,
							trip_cabin.cabin_type						AS class_name,
							trip_cabin.price + trip.surcharge			AS price_amount,
							trip_cabin.currency							AS price_currency,
							( {$count_trip_availability} )				AS trip_availability,
							trip.note_public							AS trip_notes

			FROM 			`{$this->bms_db}`.trip_cabin
				AS			trip_cabin

			JOIN 			`{$this->bms_db}`.trip
				AS			trip
				ON 			trip.id = trip_cabin.trip_id
				
			JOIN			`{$this->bms_db}`.trip_template
			  	AS			trip_template
			  	ON			trip_template.id = trip.trip_template_id

			JOIN 			`{$this->bms_db}`.trip_template_profile
				AS			trip_template_profile
				ON 			trip_template_profile.trip_template_id = trip.trip_template_id

			JOIN			`{$this->bms_db}`.boat_profile
				AS			boat_profile
				ON			boat_profile.boat_id = trip.boat_id

			JOIN			`{$this->salestool_db}`.st_vessels
				AS 			vessels
				ON			vessels.ves_name = boat_profile.name

			WHERE			trip_template.itinerary_type = 'ITINERARY'
				AND			boat_profile.name = :vessel_name
				AND			trip.date_from >= :start_date
				AND			trip.date_from >= vessels.ves_first_published_date
				AND			trip.date_from <= :end_date
				AND			trip.date_from <= vessels.ves_latest_published_date
				AND			trip.status = 'ACTIVE'
				

			GROUP BY 		trip_code, class_name

			ORDER BY		trip_start_date

			";

		$sql = $this->prepare( $statement );

		$a_params = array(
			":vessel_name"		=> $vessel_info->name,
			":start_date"		=> $start_date,
			":end_date"			=> $end_date
			);

		try {
			$sql->execute( $a_params );
			$a_trips = array();
			
			while( $trip = $sql->fetchObject()) {
			
				//  If we don't have a record for this trip code, create a new one
				//  and add it to the $a_trips array.
				//
				if ( !( isSet( $a_trips[ $trip->trip_code ] ))) {
					$trip_object = new tripClass( $trip, $a_classes );
					$a_trips[ $trip->trip_code ] = $trip_object;
				}
			
				//  Update the trip cabin information with the details for this
				//  class.
				//
				$a_trips[ $trip->trip_code ]->trip_prices[ $trip->class_name ]->price_amount = $trip->price_amount;
			
			}
		}
			
		catch( PDOException $e ) {
			$a_trips = false;
		}

		return( $a_trips );
	}




	/**
	 * Function		: getVesselTripsFromPaxtool()
	 *
	 * @param $vessel_info
	 * @param $start_date
	 * @param $end_date
	 * @param $classes
	 * @param $currency
	 * @return array
	 *
	 * Get schedule and availability for vessels that are handled in Paxtool.
	 *
	 */
	function getVesselTripsFromPaxtoolForBO2(
			$vessel_info,
			$start_date,
			$end_date,
			$classes,
			$currency_preferred='USD'
			){
		
			//  For right now we just use USD for all these vessels, no matter what is 
			//  preferred by the user.
			//
			$currency_preferred = 'USD';
			
			$curr_user 		= strtolower( $currency_preferred );
			$curr_default 	= strtolower( $currency_preferred );

			//  Build an empty remplate array of classes available during this
			//  period.  This allows us to have "N/A" entries in the price list
			//  when cabin classes are unavailable on that trip.
			//
			$a_active_cabin_classes = $this->getActiveCabinClasses( $vessel_info->name, $start_date, $end_date );

			//  Paxtool agency ID of the agency that is used as a 'BO2 - BLOCK' to block out spaces that
			//  are not booked by WWDAS but which we do not know are open or not.
			//
			$hold_agent_id = '1103';

	    	//  Sub query to count the number of spaces available in a certain class.
	    	//
	        $count_spaces_available = "
	                        SELECT 		COUNT(*)

	                        FROM		`{$this->paxtool_db}`.pax_si_belegung_details
	                          AS		beld

	                        JOIN		`{$this->paxtool_db}`.pax_si_belegung
	                          AS		bel
	                          ON		bel.bel_ter_key = beld.beld_ter_key
	                            AND		bel.bel_platz_nr = beld.beld_platz_nr

	                        LEFT JOIN `{$this->paxtool_db}`.pax_si_agenten
	                          AS      agt
	                          ON      agt.agt_key = beld.beld_agt_key

	                        JOIN		`{$this->paxtool_db}`.pax_si_termine
	                          AS		ter2
	                          ON		ter2.ter_key = beld.beld_ter_key

	                        JOIN		`{$this->salestool_db}`.ml_vessels
	                          AS		vessels2
	                          ON		vessels2.vessel_name = ter2.ter_schiff

	                        JOIN		`{$this->salestool_db}`.ml_space_classes
	                          AS		space_classes
	                          ON		space_classes.sc_vessel_id = vessels2.vessel_key
	                            AND		space_classes.sc_space = beld.beld_platz_nr

	                        #  Include records where the booking status is 'available' or the spaces_available
	                        #  is held by agency 'BO2 - BLOCK' and is not confirmed.
	                        #
	                        WHERE		beld.beld_ter_key = ter.ter_key
	                          AND   space_classes.sc_class_id = classes.class_key
	                          AND		( beld.beld_buchungsstatus = 'available'
	                                  OR beld.beld_is_buyback = 'Yes'
	                                  OR (  beld.beld_agt_flag = 'AGT' AND
	                                        beld.beld_agt_key = :space_hold_agent_id AND
	                                        beld.beld_buchungsstatus != 'confirmed'
	                                        )
	                                  )
	                          AND		bel.bel_kabine_status = 'public'
	                        ";

	        //  blue o two vessels only use 'special prices' since the prices change frequently and
	        //  there is only one class per vessel.  We therefore need to know which prices
	        //  are currently valid.
	        //
			$valid_special_prices = "
        		        SELECT            most_recent_prices.*
        
        		        FROM            (
        		        			SELECT            special_prices.*
        		        
        		        		        FROM              `{$this->salestool_db}`.ml_special_prices
        		        		          AS              special_prices
        		        
        		        		        LEFT JOIN         `{$this->salestool_db}`.ml_special_prices
        		        		          AS              most_recently_effective
        		        		          ON              most_recently_effective.trip_id = special_prices.trip_id
        		        		            AND           most_recently_effective.class_id = special_prices.class_id
        		        		            AND           most_recently_effective.validity_starts > special_prices.validity_starts
        		        
        		        		        WHERE             special_prices.validity_starts <= NOW()
        		        		            AND           special_prices.validity_ends >= NOW()
        		        		            AND           most_recently_effective.id IS NULL
        
        						
        		                           )        as most_recent_prices
        
        
            			LEFT JOIN	`{$this->salestool_db}`.ml_special_prices
            			  AS		most_recently_updated
            			  ON		most_recently_updated.vessel_id = most_recent_prices.vessel_id
            			    AND		most_recently_updated.trip_id 	= most_recent_prices.trip_id
            			    AND		most_recently_updated.class_id 	= most_recent_prices.class_id
        			    AND		most_recently_updated.last_update > most_recent_prices.last_update
            
            			WHERE 		most_recently_updated.id IS NULL
            
            			ORDER BY	trip_id, class_id
		        ";

	        //  Main query.
	        //
	   		$statement = "
	            SELECT		:vessel                                 AS vessel,
							ter.ter_key								AS trip_key,
	                        ter.ter_code                            AS trip_code,
	                        ter.ter_von                             AS trip_start_date,
	                        ter.ter_bis                             AS trip_end_date,
							DATEDIFF( ter.ter_bis, ter.ter_von)		AS trip_nights,
							itins.itin_key							AS trip_itin_key,

							IF( itins.itin_name_short = '',
								itins.itin_name,
								itins.itin_name_short)				AS trip_itinerary,

							IF( itins.itin_name = '',
								TRIM( BOTH FROM itins.itin_name_short ),
								TRIM( BOTH FROM itins.itin_name )
								)									AS trip_itinerary_long,

							IF( itins.itin_embarkation_point = '',
								'TBA',
								itins.itin_embarkation_point
								)									AS trip_embarkation_point,

							IF( itins.itin_disembarkation_point = '',
								'TBA',
								itins.itin_disembarkation_point
								)									AS trip_disembarkation_point,

							( {$count_spaces_available} )			AS trip_availability,

							classes.class_key						AS price_class, 
							classes.class_name						AS class_name,

	                        IF ( special_prices.id IS NOT NULL,                     # If a special price is defined
	                            IF( special_prices.price_{$curr_user} = -1,
	                              special_prices.price_{$curr_default},
	                              special_prices.price_{$curr_user}
	                            ),
	                            IF( prices.price_{$curr_user} = -1,
	                              prices.price_{$curr_default},
	                              prices.price_{$curr_user}
	                              )
	                            )                                   AS price_amount,

	                        IF ( special_prices.id IS NOT NULL,                     # Use the currency that matches the price.
	                            IF( special_prices.price_{$curr_user} = -1,
	                              :curr_default,
	                              :curr_user
	                            ),
	                            IF( prices.price_{$curr_user} = -1,
	                              :curr_default,
	                              :curr_user
	                              )
	                            )                                   AS price_currency


	            FROM		`{$this->paxtool_db}`.pax_si_termine
	                AS	    ter

	            JOIN		`{$this->paxtool_db}`.pax_si_termine_details
	                AS	    terd
	                ON      terd.terd_key = ter.ter_key

	            JOIN		`{$this->paxtool_db}`.pax_si_routen_details
	                AS	    roud
	                ON	    roud.roud_key = ter.ter_rou_key

	            JOIN		`{$this->paxtool_db}`.pax_si_routen
	                AS	    rou
	                ON	    rou.rou_key = ter.ter_rou_key

	            LEFT JOIN	`{$this->salestool_db}`.ml_itineraries
	                AS	    itins
	                ON	    itins.itin_pax_key = roud.roud_key

	            JOIN        `{$this->salestool_db}`.st_vessels
	                AS      vessel_info
	                ON      vessel_info.ves_name = ter.ter_schiff

	            JOIN        `{$this->salestool_db}`.ml_vessels
	                AS      vessels
	                ON      vessels.vessel_name = ter.ter_schiff

	            JOIN        `{$this->salestool_db}`.ml_classes
	                AS      classes
	                ON      classes.class_vessel = vessels.vessel_key

	            LEFT JOIN	  `{$this->salestool_db}`.ml_prices
	                AS	    prices
	                ON	    prices.price_itin = itins.itin_key
	                  AND   prices.price_class = classes.class_key
	                  AND	prices.price_start_date <= ter.ter_von
	                  AND	prices.price_end_date >= ter.ter_von

	            LEFT JOIN   ( {$valid_special_prices} )
	                AS      special_prices
	                ON      special_prices.trip_id = ter.ter_key
	                  AND   special_prices.vessel_id = vessels.vessel_key
	                  AND   special_prices.class_id = classes.class_key

	            LEFT JOIN	`{$this->salestool_db}`.ml_surcharges
	                AS	    surcharges
	                ON	    surcharges.trip_id = ter.ter_key
	                  AND	  surcharges.vessel_id = vessels.vessel_key

	            WHERE		classes.class_is_public
	                AND     ter.ter_schiff = :vessel
	                AND	    ter.ter_von BETWEEN :start_date AND :end_date
	                AND     ter.ter_von BETWEEN vessel_info.ves_first_published_date AND vessel_info.ves_latest_published_date
					AND 	ter.ter_von BETWEEN vessels.vessel_start_operation AND vessels.vessel_end_operation
	                AND	    rou.rou_typ = 'I'

	            GROUP BY	ter.ter_key,
	                        classes.class_key

	            ORDER BY	vessel,
	                        trip_start_date,
	                        classes.class_sort_order
	            ";
	   		
	   		$sql = $this->prepare($statement);
	   		
			$a_params = array(
					':space_hold_agent_id'		=> $hold_agent_id,
					':vessel' 					=> $vessel_info->name,
					':start_date' 				=> $start_date,
					':end_date' 				=> $end_date,
					":curr_default"				=> strtoupper( $curr_default ),
					":curr_user"				=> strtoupper( $curr_user )
					);
		
		$fql = fopen( "/tmp/sql_query.log", "w" );
		fprintf( $fql, sprintf( "\n\n---\nParameter list:\n%s\n\n", print_r( $a_params, true )));
		fprintf( $fql, sprintf( "\n\n---\nQuery statement:\n%s\n\n", print_r( $statement, true )));
		fclose( $fql );
		
			try { 
				$sql->execute( $a_params );
				$a_data = $sql->FetchAll( PDO::FETCH_CLASS );
				
				$a_trips = array();
				
				foreach( $a_data as $data ) {
					
					if( !isSet( $a_trips[ $data->trip_key ] )) {
						$a_trips[ $data->trip_key ] = new bo2TripDataClass( $data );
					}
					
					$a_trips[ $data->trip_key ]->addPrice( $data->price_class, $data->class_name, $data->price_amount, $data->price_currency );
				}
			}
			
			catch ( PDOException $e ) {
				$a_trips = array();
			}

			return( $a_trips );
	}




	/**
	 * Function		: getVesselTripsFromPaxtool()
	 *
	 * @param $vessel_info
	 * @param $start_date
	 * @param $end_date
	 * @param $classes
	 * @param $currency
	 * @return array
	 *
	 * Get schedule and availability for vessels that are handled in Paxtool.
	 *
	 */
	function getVesselTripsFromPaxtool(
			$vessel_info,
			$start_date,
			$end_date,
			$classes,
			$currency_preferred=null
			){

			//  Build an empty remplate array of classes available during this
			//  period.  This allows us to have "N/A" entries in the price list
			//  when cabin classes are unavailable on that trip.
			//
			$a_active_cabin_classes = $this->getActiveCabinClasses( $vessel_info->name, $start_date, $end_date );


			//  If a vessel is defined, match either the name or the key.
			//
			if ( $vessel_info == "" ) {
				$vessel = 1;
				$vessel_filter = ":vessel = :vessel";
			}
			else {
				$vessel_filter = "(( vessels.vessel_key = :vessel ) OR ( vessels.vessel_name = :vessel))";
			}
			
			#  2017-01-31:DABD (RM #7610)
			#
			#  Adjust where clause to count as available if buy-back flag is set. 
			#
			$query_availability = "
					SELECT		COUNT(*)

					FROM		`{$this->paxtool_db}`.pax_si_belegung_details
						AS		beld

					JOIN		`{$this->paxtool_db}`.pax_si_belegung
						AS		bel
						ON		bel.bel_ter_key = beld.beld_ter_key
						  AND	bel.bel_platz_nr = beld.beld_platz_nr
  
					WHERE		beld.beld_ter_key = ter.ter_key 
						AND		( beld.beld_buchungsstatus = 'available' OR beld.beld_is_buyback = 'Yes' )
						AND		bel.bel_kabine_status = 'public'
					";

			//  Get a list of trips.
			//
			//
			//  2017-01-24:DABD (RM #7561)
			//		Updated to use long itinerary name if the short one is blank and to use the short
			//		itin name if the long one is blank.  Also will use 'TBA' for either embarkation
			//		or disembarkation points if either is blank.
			//
			$sql_query = "

					SELECT			vessels.vessel_name										AS vessel_name,
									ter.ter_key												AS trip_key,
									ter.ter_code											AS trip_code,
									ter.ter_von												AS trip_start_date,
									ter.ter_bis												AS trip_end_date,
									DATEDIFF( ter.ter_bis, ter.ter_von)					 	AS trip_nights,

									IF( itins.itin_name_short = '',
										itins.itin_name,
										itins.itin_name_short)								AS trip_itinerary,

									IF( itins.itin_name = '',
										itins.itin_name_short,
										itins.itin_name
										)													AS trip_itinerary_long,

									IF( itins.itin_embarkation_point = '',
										'TBA',
										itins.itin_embarkation_point
										)													AS trip_embarkation_point,

									IF( itins.itin_disembarkation_point = '',
										'TBA',
										itins.itin_disembarkation_point
										)													AS trip_disembarkation_point,

									'TBA'													AS trip_prices,
									( {$query_availability} )								AS trip_availability,

									itins.itin_key											AS trip_itin_key,
									terd.terd_bemerkung_public								AS trip_notes
									
									
					FROM			{$this->paxtool_db}.pax_si_termine
						AS			ter
						
					JOIN			{$this->paxtool_db}.pax_si_termine_details
					  	AS			terd
					  	ON			terd.terd_key = ter.ter_key
						
					JOIN			{$this->paxtool_db}.pax_si_routen
						AS			rou
						ON			rou.rou_key = ter.ter_rou_key

					JOIN			{$this->paxtool_db}.pax_si_routen_details
						AS			roud
						ON			roud.roud_key = ter.ter_rou_key

					JOIN			{$this->schedule_db}.ml_vessels
						AS			vessels
						ON			vessels.vessel_name = ter.ter_schiff

					LEFT JOIN		{$this->salestool_db}.st_vessels
						AS 			st_vessels
						ON 			st_vessels.ves_name = ter.ter_schiff

					JOIN			{$this->schedule_db}.ml_itineraries
						AS			itins
						ON			itins.itin_pax_key = ter.ter_rou_key

					WHERE			{$vessel_filter}
						AND 		ter.ter_von >= :start_date
						AND 		ter.ter_von >= vessels.vessel_start_operation
						AND 		ter.ter_von >= st_vessels.ves_first_published_date
						AND			ter.ter_von  <= :end_date
						AND			rou.rou_typ = 'I'

					ORDER BY		trip_start_date;

					";

			$sql = $this->prepare( $sql_query );

			$a_params = array(
					':vessel' 		=> $vessel_info->name,
					':start_date' 	=> $start_date,
					':end_date' 	=> $end_date
					);

			try {
				$sql->execute( $a_params );
				$a_trips = $sql->fetchAll( PDO::FETCH_OBJ );

				foreach( $a_trips as $trip ) {
					$a_prices = $this->getTripPrices( 
							$trip->vessel_name, 
							$trip->trip_key,
							$trip->trip_itin_key,
							$trip->trip_start_date,
							$trip->trip_end_date
							);

					$trip->trip_prices = $this->newEmptyClassTable( $a_active_cabin_classes );

					foreach( $a_prices as $price ) {
						$trip->trip_prices[ $price[ 'class_name' ]]->price_amount = $price[ 'price_amount' ];
						$trip->trip_prices[ $price[ 'class_name' ]]->price_currency = $price[ 'price_currency' ];
					}
				}
			}
			catch ( PDOException $e ) {
				echo "ERROR: SQL query failed.  " . $e->getMessage();
				$a_trips = array();
			}
			
			return( $a_trips );
		}


		/**
		 * Function		: newEmptyClassTable()
		 *
		 * @param unknown $a_active_cabin_classes
		 * @return stdClass[]
		 *
		 * Creates a new empty object to handle class data.  Added to allow a complete
		 * matrix even if come classes are not specified -- ie: on some Truk Master
		 * trips we do not sell some classes so there is no pricing or availability
		 * available for that class, therefore the class has no data.
		 *
		 */
		function newEmptyClassTable( $a_active_cabin_classes ) {

			$empty_class_table = array();

			foreach( $a_active_cabin_classes as $cabin_class ) {

				$class_info = new stdClass();

				$class_info->price_class = $cabin_class->class_key;
				$class_info->class_name = $cabin_class->class_name;
				$class_info->price_amount = "N/A";
				$class_info->price_currency = "";

				$empty_class_table[ $cabin_class->class_name ] = $class_info;

			}

			return( $empty_class_table );
		}



		/**
		 * Function		: getTripPrices()
		 *
		 * @param unknown $vessel_name
		 * @param unknown $trip_itin_key
		 * @param unknown $trip_key
		 * @param unknown $start_date
		 * @param unknown $end_date
		 *
		 * Get the prices for each class for a given trip.
		 *
		 */
		function getTripPrices( $vessel_name, $trip_key, $trip_itin_key,  $trip_start_date=null, $trip_end_date=null ) {

			static $sql;		// Set this as static so we only have to prepare it first time.

			//  If the SQL statement is not prepared, do that now.  Will only be done the first time through.
			//
			if( !( isSet( $sql ))) {
				//  For right now assume USD for all prices -- use lower case so it matches the field name.
				//
				$cur = "usd";

				$sql_query = "
					SELECT				prices.price_class								AS price_class,
										classes.class_name								AS class_name,

										#  If there is a special price set for this trip, use it
										#  else use the normal price.  If there is a surcharge
										#  defined, use it.
										#
										IF( special_prices.id IS NOT NULL,
											special_prices.price_{$cur},
											prices.price_{$cur}
											) +
										IF ( surcharges.id IS NULL,
											0,
											surcharges.amount_{$cur}
											)											AS price_amount,

										'USD'											AS price_currency

										FROM			`{$this->paxtool_db}`.pax_si_termine
										  AS			ter

										JOIN			`{$this->schedule_db}`.ml_itineraries
										  AS			itins
										  ON			itins.itin_pax_key = ter.ter_rou_key

										JOIN			`{$this->schedule_db}`.ml_vessels
										  AS			vessels
										  ON			vessels.vessel_name = ter.ter_schiff

										JOIN			`{$this->schedule_db}`.ml_classes
										  AS			classes
										  ON			classes.class_vessel = vessels.vessel_key

										JOIN			`{$this->schedule_db}`.ml_prices
										  AS			prices
										  ON			prices.price_itin = itins.itin_key
										    AND			prices.price_class = classes.class_key

										LEFT JOIN		`{$this->schedule_db}`.ml_surcharges
										  AS			surcharges
										  ON			surcharges.trip_id = ter.ter_key

										LEFT JOIN		`{$this->schedule_db}`.ml_special_prices
										  AS			special_prices
										  ON			special_prices.trip_id = ter.ter_key
										    AND			special_prices.class_id = classes.class_key

										WHERE			ter.ter_schiff = :vessel
										  AND			ter.ter_key = :trip_key
										  AND			:trip_start_date BETWEEN prices.price_start_date AND prices.price_end_date

										ORDER BY		classes.class_sort_order
										";

				$sql = $this->prepare( $sql_query );
				}

				$a_params = array(
						":vessel"				=> $vessel_name,
						":trip_start_date"		=> $trip_start_date,
						":trip_key"				=> $trip_key
						);

				try {
					$sql->execute( $a_params );
					$a_prices = $sql->fetchAll( PDO::FETCH_ASSOC );
				}
				catch ( PDOException $e ) {
					echo "\nERROR: Unable to retrieve trip pricing.\n\n{$e}\n\n";
				}

				return( $a_prices );
		}



		function getTripAvailability( $trip_code ) {

			//  Sub query to count the number of available spaces.
			//
			$sql_count_spaces = "

				SELECT 			COUNT(*)

				FROM			{$this->schedule_db}.pax_si_belegung_details
					AS			beld

				JOIN			{$this->schedule_db}.pax_si_belegung
					AS			bel
					ON			(( bel.bel_ter_key = beld.beld_ter_key ) AND ( bel.bel_platz_nr = beld.beld_platz_nr ))

				WHERE			beld.beld_ter_key 	= ter.ter_key
					AND			beld.beld_buchungsstatus = 'available'
					AND			bel.bel_kabine_status = 'public'

				";

			//  Main query to get the list of trips and the number of spaces
			//  available on each.
			//
			$sql_query = "

					SELECT		ter.ter_code							AS trip_code,
								ter.ter_schiff							AS trip_vessel,
								roud.roud_name							AS trip_itinerary,
								ter.ter_von								AS trip_start_date,
								ter.ter_bis								AS trip_end_date,
								DATEDIFF( ter.ter_bis, ter.ter_von )	AS trip_nights,
								ter.ter_preis_1_wert					AS trip_price_eur,

								( {$sql_count_spaces} ) 				AS trip_spaces_available,

								terd.terd_bemerkung_public				AS trip_notes

					FROM		{$this->schedule_db}.pax_si_termine
						AS		ter

					JOIN		{$this->schedule_db}.pax_si_termine_details
						AS		terd
						ON		terd.terd_key = ter.ter_key

					JOIN		{$this->schedule_db}.pax_si_routen_details
						AS		roud
						ON		roud.roud_key = ter.ter_rou_key

					WHERE		ter.ter_code = :trip_code;
					";

			$sql_vars = array( ":trip_code" => $trip_code );

			$sql = $this->prepare( $sql_query );
			$sql->execute( $sql_vars );

			$a_results = $sql->fetchAll( PDO::FETCH_OBJ );

			$availability = ( count( $a_results ) == 0 )
				? FALSE
				: $a_results[ 0 ];

			return( $availability );
		}

	}
	?>
